==> modal-0.0.1.tgz: package/index.js <==
import { zIndexManager } from "./util/zIndexManager";

export class Modal {
  static #zIndexManager = new zIndexManager();

  parents = "";
  html = "";
  modalClass = "";
  contentsClass = "";
  backgroundClass = "";
  transitionSeconds = 0.3;
  backgroundClickExit = true;

  beforeOpen = () => {};
  afterOpen = () => {};
  beforeEnd = () => {};
  afterEnd = () => {};

  constructor(parents, html) {
    if (parents === null) throw "parents must be entered";
    if (html === null) throw "html must be entered";

    this.parents = parents;
    this.html = html;
  }

  async open() {
    // create id
    const id = String(Modal.makeID(32));

    // get z-index
    const zIndex = Modal.getZIndex(this.parents);

    // set modal id
    const modalId = "modal_" + id;

    // add default modal css
    if (document.getElementById("frontleModalCSS") === null) {
      let modalCSSElement = document.createElement("style");
      modalCSSElement.setAttribute("id", "frontleModalCSS");
      modalCSSElement.innerHTML = `
          .frontleModal{
              position: fixed;
              width: 100%;
              height: 100%;
              top: 0;
              left: 0;
          }
          .frontleModalBackground{
              position: relative;
              width: 100%;
              height: 100%;
              top: 0;
              left: 0;
              background: rgba(0, 0, 0, 0.4);
          }
          .frontleModalContents{
              position: absolute;
              width: 200px;
              height: 80px;
              top: 50%;
              left: 50%;
              transform: translate(-50%, -50%);  
              box-sizing: border-box;
              background: #ffffff;
              overflow: hidden;
          }
      `;
      document.head.insertBefore(modalCSSElement, document.head.childNodes[0]);
    }

    // set html
    const html = /* html */ `
    <div class="frontleModalBackground ${
      this.backgroundClass
    }" style="z-index: ${zIndex + 1}"></div>
      
    <div class="frontleModalContents ${this.contentsClass}" style="z-index: ${
      zIndex + 2
    }">${this.html}</div>
    `;

    // add modal
    let modalElement = document.createElement("div");
    modalElement.setAttribute("id", modalId);
    modalElement.className = `frontleModal ${this.modalClass}`;
    modalElement.innerHTML = html;
    modalElement.style.zIndex = String(zIndex);
    modalElement.style.opacity = "0";
    modalElement.style.transition = `opacity ease ${this.transitionSeconds}s 0s`;
    document.getElementById(this.parents).append(modalElement);

    // run lifecycle
    await this.beforeOpen();

    // set close event
    if (this.backgroundClickExit === true) {
      document
        .querySelector(modalId)
        .querySelector(".frontleModalBackground")
        .addEventListener("click", () => {
          this.close();
        });
    }

    // start modal animation
    setTimeout(() => {
      let modal = document.getElementById(modalId);
      if (modal !== null) modal.style.opacity = "1";
    }, 0);

    // end modal animation
    setTimeout(async () => {
      // run lifecycle
      await this.afterOpen();
    }, Number(this.transitionSeconds) * 1000);

    return modalId;
  }

  async close(modalId) {
    // run lifecycle
    await this.beforeEnd();

    // start modal animation
    let modalElement = document.getElementById(modalId);
    if (modalElement !== null) modalElement.style.opacity = "0";

    // end modal animation
    setTimeout(async () => {
      // remove modal
      if (modalElement !== null) modalElement.remove();

      // run lifecycle
      await this.afterEnd();
    }, Number(this.transitionSeconds) * 1000);
  }

  static getZIndex(parents) {
    return this.#zIndexManager.getHighestZIndex(parents);
  }

  static makeID(length) {
    let result = "";
    let characters =
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    let charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
  }
}

==> modal-0.0.2.tgz: package/index.js <==
import { zIndexManager } from "./util/zIndexManager.js";

export class Modal {
  static #zIndexManager = new zIndexManager();

  parents = "";
  html = "";
  modalClass = "";
  contentsClass = "";
  backgroundClass = "";
  transitionSeconds = 0.3;
  backgroundClickExit = true;

  beforeOpen = () => {};
  afterOpen = () => {};
  beforeEnd = () => {};
  afterEnd = () => {};

  constructor(parents, html) {
    if (parents === null) throw "parents must be entered";
    if (html === null) throw "html must be entered";

    this.parents = parents;
    this.html = html;
  }

  async open() {
    // create id
    const id = String(Modal.makeID(32));

    // get z-index
    const zIndex = Modal.getZIndex(this.parents);

    // set modal id
    const modalId = "modal_" + id;

    // add default modal css
    if (document.getElementById("frontleModalCSS") === null) {
      let modalCSSElement = document.createElement("style");
      modalCSSElement.setAttribute("id", "frontleModalCSS");
      modalCSSElement.innerHTML = `
          .frontleModal{
              position: fixed;
              width: 100%;
              height: 100%;
              top: 0;
              left: 0;
          }
          .frontleModalBackground{
              position: relative;
              width: 100%;
              height: 100%;
              top: 0;
              left: 0;
              background: rgba(0, 0, 0, 0.4);
          }
          .frontleModalContents{
              position: absolute;
              width: 200px;
              height: 80px;
              top: 50%;
              left: 50%;
              transform: translate(-50%, -50%);  
              box-sizing: border-box;
              background: #ffffff;
              overflow: hidden;
          }
      `;
      document.head.insertBefore(modalCSSElement, document.head.childNodes[0]);
    }

    // set html
    const html = /* html */ `
    <div class="frontleModalBackground ${
      this.backgroundClass
    }" style="z-index: ${zIndex + 1}"></div>
      
    <div class="frontleModalContents ${this.contentsClass}" style="z-index: ${
      zIndex + 2
    }">${this.html}</div>
    `;

    // add modal
    let modalElement = document.createElement("div");
    modalElement.setAttribute("id", modalId);
    modalElement.className = `frontleModal ${this.modalClass}`;
    modalElement.innerHTML = html;
    modalElement.style.zIndex = String(zIndex);
    modalElement.style.opacity = "0";
    modalElement.style.transition = `opacity ease ${this.transitionSeconds}s 0s`;
    document.getElementById(this.parents).append(modalElement); // TODO querySelector

    // run lifecycle
    await this.beforeOpen();

    // set close event
    if (this.backgroundClickExit === true) {
      document
        .querySelector(modalId)
        .querySelector(".frontleModalBackground")
        .addEventListener("click", () => {
          this.close();
        });
    }

    // start modal animation
    setTimeout(() => {
      let modal = document.getElementById(modalId);
      if (modal !== null) modal.style.opacity = "1";
    }, 0);

    // end modal animation
    setTimeout(async () => {
      // run lifecycle
      await this.afterOpen();
    }, Number(this.transitionSeconds) * 1000);

    return modalId;
  }

  async close(modalId) {
    // run lifecycle
    await this.beforeEnd();

    // start modal animation
    let modalElement = document.getElementById(modalId);
    if (modalElement !== null) modalElement.style.opacity = "0";

    // end modal animation
    setTimeout(async () => {
      // remove modal
      if (modalElement !== null) modalElement.remove();

      // run lifecycle
      await this.afterEnd();
    }, Number(this.transitionSeconds) * 1000);
  }

  static getZIndex(parents) {
    return this.#zIndexManager.getHighestZIndex(parents);
  }

  static makeID(length) {
    let result = "";
    let characters =
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    let charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
  }
}
